@@ -7,26 +7,25 @@ setGeneric (
     def = function (x, table.name = NULL, verbose = TRUE, ...) {
         x.str <- deparse(substitute(x))
         res <- standardGeneric("as.db.data.frame")
+        tst <- paste(res$res@.name, collapse = ".")
         if (verbose) {
             if (is.data.frame(x)) {
                 cat("\nThe data in the data.frame", x.str,
-                    "is stored into the table", table.name, "in database",
+                    "is stored into", tst, "in database",
                     dbname(res$conn.id), "on", host(res$conn.id), "!\n")
             } else if (is.character(x)) {
                 cat("\nThe data in the file", x.str,
-                    "is stored into the table", table.name, "in database",
+                    "is stored into", tst, "in database",
                     dbname(res$conn.id), "on", host(res$conn.id), "!\n")
             } else if (is(x, "db.Rquery")) {
                 cat("\nThe data created by", x.str,
-                    "is stored into the table", table.name, "in database",
+                    "is stored into", tst, "in database",
                     dbname(res$conn.id), "on", host(res$conn.id), "!\n\n")
             } else {
                 cat("\nThe data contained in table", content(x),
                     "which is wrapped by", x.str,
-                    "is copied into the table", table.name, "in database",
+                    "is copied into", tst, "in database",
                     dbname(res$conn.id), "on", host(res$conn.id), "!\n\n")
-                cat("An R object pointing to", table.name,
-                    "in connection", conn.id(x), "is created !\n")
             }
         }
 
@@ -42,7 +41,7 @@ setMethod (
     signature (x = "data.frame"),
     def = function (
     x, table.name = NULL, verbose = TRUE, conn.id = 1, add.row.names = FALSE,
-    key = character(0), distributed.by = NULL,
+    key = character(0), distributed.by = NULL, append = FALSE,
     is.temp = FALSE, ...) {
         if (is.null(table.name)) {
             table.name <- .unique.string()
@@ -51,7 +50,7 @@ setMethod (
         .method.as.db.data.frame.1(x,
                                    table.name, verbose, conn.id,
                                    add.row.names, key,
-                                   distributed.by, is.temp, ...)
+                                   distributed.by, append, is.temp, ...)
     })
 
 ## -----------------------------------------------------------------------
@@ -63,7 +62,7 @@ setMethod (
     signature (x = "character"),
     def = function (
     x, table.name = NULL, verbose = TRUE, conn.id = 1, add.row.names = FALSE,
-    key = character(0), distributed.by = NULL,
+    key = character(0), distributed.by = NULL, append = FALSE,
     is.temp = FALSE, ...) {
         if (is.null(table.name)) {
             table.name <- .unique.string()
@@ -77,14 +76,14 @@ setMethod (
         .method.as.db.data.frame.1(x,
                                    table.name, verbose, conn.id,
                                    add.row.names, key,
-                                   distributed.by, is.temp, ...)
+                                   distributed.by, append, is.temp, ...)
     })
 
 ## -----------------------------------------------------------------------
 
 .method.as.db.data.frame.1 <- function (
     x, table.name = NULL, verbose = TRUE, conn.id = 1, add.row.names = FALSE,
-    key = character(0), distributed.by = NULL,
+    key = character(0), distributed.by = NULL, append = FALSE,
     is.temp = FALSE, ...)
 {
     if (!.is.conn.id.valid(conn.id))
@@ -98,8 +97,15 @@ setMethod (
     exists <- db.existsObject(table.name, conn.id, is.temp)
     if (is.temp) exists <- exists[[1]]
     if (exists) {
-        .restore.warnings(warnings)
-        stop("The table already exists in connection ", conn.id, "!")
+        if (!append) {
+            .restore.warnings(warnings)
+            stop("The table already exists in connection ", conn.id, "!")
+        }
+    } else {
+        if (append) {
+            .restore.warnings(warnings)
+            stop("The table to be appended does not exist in connection ", conn.id, "!")
+        }
     }
 
     if (!.is.arg.string(key)) stop("ID column name must be a string!")
@@ -123,14 +129,15 @@ setMethod (
     ## if (missing(is.temp)) is.temp <- FALSE
 
     table <- .db.analyze.table.name(table.name)
-    if ((!is.temp && .db.existsTable(table, conn.id)) ||
-        (is.temp && .db.existsTempTable(table, conn.id)[[1]])) {
+
+    if (!append && ((!is.temp && .db.existsTable(table, conn.id)) ||
+        (is.temp && .db.existsTempTable(table, conn.id)[[1]]))) {
         .restore.warnings(warnings)
         stop("Table already exists!")
     }
 
     .db.writeTable(table, x, add.row.names = add.row.names,
-                   distributed.by = distributed.by,
+                   distributed.by = distributed.by, append = append,
                    is.temp = is.temp, conn.id = conn.id, ...)
 
     if (length(table) == 1 && !is.temp) {
@@ -163,6 +170,10 @@ setMethod (
 
 ## -----------------------------------------------------------------------
 
+## Use this to represent NULL
+## user is extremely unlikely to use this name
+.null.string <- "<@#$NULL%^&>"
+
 ## convert a db.Rquery object into a db.data.frame object
 
 setMethod (
@@ -172,6 +183,7 @@ setMethod (
     is.view = FALSE,
     is.temp = FALSE,  pivot = TRUE,
     distributed.by = NULL, nrow = NULL, field.types = NULL,
+    na.as.level = FALSE, # whether use NULL as a level
     factor.full = rep(FALSE, length(names(x)))) { # whether expand all levels
         warnings <- .suppress.warnings(conn.id(x))
 
@@ -254,24 +266,16 @@ setMethod (
         factor.ref <- rep(as.character(NA), length(x@.is.factor))
         if (pivot && !all(x@.is.factor == FALSE)) {
             cats <- x@.expr[x@.is.factor]
-            sql <- "select "
-            for (i in seq_len(length(cats))) {
-                sql <- paste(sql, "array_agg(distinct case when ", cats[i],
-                             " is NULL then 'NULL' else ",
-                             cats[i], "::text end) as ",
-                             "distinct_", i, sep = "")
-                if (i != length(cats)) sql <- paste(sql, ",", sep = "")
-            }
-            ## scan through the table only once
-            sql <- paste(sql, " from ", tbl, where, sep = "")
-            distincts <- .db.getQuery(sql, conn.id)
-            idx <- 0
             for (i in seq_len(length(x@.is.factor))) {
                 if (x@.is.factor[i]) {
-                    idx <- idx + 1
-                    distinct <- as.vector(arraydb.to.arrayr(distincts[[paste("distinct_",idx,sep="")]], "character"))
-                    ## Produce a fixed order for distinct values
-                    distinct <- distinct[order(distinct, decreasing = TRUE)]
+                    distinct <- lk(by(x[[i]], x[[i]], identity))[,1]
+                    if (na.as.level)
+                        distinct[is.na(distinct)] <- .null.string
+                    else
+                        distinct <- distinct[!is.na(distinct)]
+
+                    distinct <- .strip(distinct[order(distinct, decreasing = TRUE)], "\"")
+
                     if (is.na(x@.factor.ref[i]))
                         avoid <- distinct[length(distinct)]
                     else
@@ -284,13 +288,14 @@ setMethod (
                         is.factor <- c(is.factor, FALSE)
                         factor.ref <- c(factor.ref, as.character(NA))
                         if (extra != "") extra <- paste(extra, ", ")
-                        dex <- paste("(case when ", x@.expr[i], "::text = '",
+                        dex <- paste("(case when (", x@.expr[i], ")::text = '",
                                      distinct[j], "'",
                                      " then 1 else 0 end)", sep = "")
                         extra <- paste(extra, " ", dex, " as ",
                                        "\"", new.col, "\"", sep = "")
                         appear <- c(appear, paste(x@.col.name[i],".",
-                                                  distinct[j], sep = ""))
+                                                  if (distinct[j] != .null.string) distinct[j]
+                                                  else "<NA>", sep = ""))
                         dummy <- c(dummy, new.col)
                         dummy.expr <- c(dummy.expr, dex)
                     }
