@@ -104,7 +104,9 @@ setMethod (
             dist.str <- paste("distributed by (", id.col, ")", sep = "")
             dist.by <- id.col
         } else {
-            dist.str <- paste("distributed by (", dist.cols, ")", sep = "")
+            dist.str <- paste("distributed by (",
+                              paste("\"", dist.cols, "\"", sep = ""),
+                              ")", sep = "")
         }
     } else {
         dist.str <- ""
