@@ -30,7 +30,7 @@
     for (i in seq(along=.supported.connections))
         .localVars$conn.type[[tolower(.supported.connections[i])]] <- integer(0)
 
-    packageStartupMessage("\nTo launch the graphical user interface, run the function pivotalr() !")
+    #packageStartupMessage("\nTo launch the graphical user interface, run the function pivotalr() !")
 
     ## turn on cbind implementation using cbind2
     .localVars$R.ver <- gsub("^R version (\\d+\\.\\d+).*$", "\\1", R.version.string)
